#!/usr/bin/env python
# -*- coding: utf-8 -*-

#----------------------------------------------------------------------------------------------------------------------------------
# includes

# 2+3 compat
from __future__ import absolute_import, division, print_function, unicode_literals

# standards
import re
import xml.etree.ElementTree

# klon
from klon.compatibility import text_type

#----------------------------------------------------------------------------------------------------------------------------------

class Klon:

    @classmethod
    def build_etree(cls, tag, *args):
        if not isinstance(tag, text_type):
            raise ValueError('Tag must be a %s, got %r' % (text_type.__name__, tag))
        attrib, args = cls._compile_attrib(*args)
        tag, attrib = cls._parse_css_style_tags(tag, attrib)
        element = cls._create_element(tag, attrib)
        cls._append_children(element, args)
        return element

    @classmethod
    def _compile_attrib(cls, *args):
        attrib = {}
        if len(args) > 0 and isinstance(args[0], dict):
            attrib.update(args[0])
            args = args[1:]
        return attrib, args

    @classmethod
    def _parse_css_style_tags(cls, tag, attrib):
        match = re.search(r'^(.+?)(\#|\.)(.+)', tag)
        if match:
            tag, key, value = match.groups()
            key = {'#': 'id', '.': 'class'}[key]
            attrib[key] = value
        return tag, attrib

    @classmethod
    def _append_children(cls, element, args):
        text_anchor = None
        for child in args:
            if child in (None, (), []):
                pass
            elif isinstance(child, text_type):
                if text_anchor is None:
                    element.text = (element.text or '') + child
                else:
                    text_anchor.tail = (text_anchor.tail or '') + child
            else:
                if not cls._is_element(child):
                    child = cls.build_etree(*child)
                element.append(child)
                text_anchor = child

    # The rest are overriden in LxmlKlon to use lxml.etree instead of xml.etree.ElementTree

    @classmethod
    def _create_element(cls, tag, attrib):
        return xml.etree.ElementTree.Element(tag, attrib)

    @classmethod
    def _is_element(cls, obj):
        return isinstance(obj, xml.etree.ElementTree.Element)

    @classmethod
    def tostring(cls, etree, **kwargs):
        return xml.etree.ElementTree.tostring(etree, **kwargs)

#----------------------------------------------------------------------------------------------------------------------------------



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#----------------------------------------------------------------------------------------------------------------------------------
# includes

# 2+3 compat
from __future__ import absolute_import, division, print_function, unicode_literals

# standards
import re

# 3rd parties
import pytest

# klon
from klon import Klon
from klon.lxml import LxmlKlon

#----------------------------------------------------------------------------------------------------------------------------------

def parametrize_implementations(body):
    parametrize = pytest.mark.parametrize('klon', [Klon, LxmlKlon])
    return parametrize(body)


def check(element, tag, attrib={}, children=[], text=None, tail=None):
    assert element.tag == tag
    assert element.attrib == attrib
    assert element.text == text
    assert len(element) == len(children)
    for child, child_definition in zip(element, children):
        check(child, **child_definition)
    assert element.tail == tail


def check_str(klon, element, expected_str):
    generated_bytes = klon.tostring(element, encoding='UTF-8')
    generated_str = generated_bytes.decode('UTF-8')
    assert _normalise_str(generated_str) == _normalise_str(expected_str)

def _normalise_str(text):
    # Smoothe over the slight cosmetic differences btw the output of xml.etree.ElementTree.tostring and lxml.etree.tostring
    text = re.sub(r'^<\?xml[^>]*>\s*', '', text)
    text = re.sub(r' (?=/>)', '', text)
    return text

#----------------------------------------------------------------------------------------------------------------------------------

@parametrize_implementations
def test_build_empty_node(klon):
    element = klon.build_etree('mynode')
    check(element, 'mynode')
    check_str(klon, element, '<mynode />')


@parametrize_implementations
def test_build_attrib_from_dict(klon):
    element = klon.build_etree('mynode', {'a': '1'})
    check(element, 'mynode', {'a': '1'})
    check_str(klon, element, '<mynode a="1" />')

#----------------------------------------------------------------------------------------------------------------------------------

@parametrize_implementations
def test_css_style_id(klon):
    element = klon.build_etree('mynode#test')
    check(element, 'mynode', {'id': 'test'})
    check_str(klon, element, '<mynode id="test" />')


@parametrize_implementations
def test_css_style_class(klon):
    element = klon.build_etree('mynode.test')
    check(element, 'mynode', {'class': 'test'})
    check_str(klon, element, '<mynode class="test" />')

#----------------------------------------------------------------------------------------------------------------------------------

@parametrize_implementations
def test_build_node_with_text(klon):
    element = klon.build_etree('mynode', 'Text')
    check(element, 'mynode', text='Text')
    check_str(klon, element, '<mynode>Text</mynode>')


@parametrize_implementations
def test_build_node_with_multiple_text(klon):
    element = klon.build_etree('mynode', 'Te', 'xt')
    check(element, 'mynode', text='Text')
    check_str(klon, element, '<mynode>Text</mynode>')


@parametrize_implementations
def test_build_node_with_text_and_attrib(klon):
    element = klon.build_etree('mynode', {'a': '1'}, 'Te', 'xt')
    check(element, 'mynode', attrib={'a': '1'}, text='Text')
    check_str(klon, element, '<mynode a="1">Text</mynode>')

#----------------------------------------------------------------------------------------------------------------------------------

@parametrize_implementations
def test_node_with_children(klon):
    element = klon.build_etree('mynode', ['mychild'])
    check(element, 'mynode', children=[{'tag': 'mychild'}])
    check_str(klon, element, '<mynode><mychild /></mynode>')


@parametrize_implementations
def test_node_with_children_attribs(klon):
    element = klon.build_etree('mynode', ['mychild', {'a': '1'}])
    check(element, 'mynode', children=[{'tag': 'mychild', 'attrib': {'a': '1'}}])
    check_str(klon, element, '<mynode><mychild a="1" /></mynode>')


@parametrize_implementations
def test_node_with_children_text(klon):
    element = klon.build_etree('mynode', ['mychild', {'a': '1'}, 'Text'])
    check(element, 'mynode', children=[{'tag': 'mychild', 'attrib': {'a': '1'}, 'text': 'Text'}])
    check_str(klon, element, '<mynode><mychild a="1">Text</mychild></mynode>')


@parametrize_implementations
def test_node_with_children_tail(klon):
    element = klon.build_etree('mynode', ['mychild'], 'Tail')
    check(element, 'mynode', children=[{'tag': 'mychild', 'tail': 'Tail'}])
    check_str(klon, element, '<mynode><mychild />Tail</mynode>')


@parametrize_implementations
def test_node_with_prebuilt_child(klon):
    child = klon.build_etree('mychild')
    element = klon.build_etree('mynode', child)
    check(element, 'mynode', children=[{'tag': 'mychild'}])
    check_str(klon, element, '<mynode><mychild /></mynode>')

#----------------------------------------------------------------------------------------------------------------------------------

@parametrize_implementations
def test_non_str_tag(klon):
    with pytest.raises(ValueError):
        klon.build_etree(object(), {'a': '1'})

@parametrize_implementations
def test_byte_tag(klon):
    with pytest.raises(ValueError):
        klon.build_etree(b'mytag', {'a': '1'})

@parametrize_implementations
def test_empty_children_array_are_noop(klon):
    element = klon.build_etree('mynode', ['mychild', 'a', [], 'b'], [], 'c')
    check(element, 'mynode', children=[{'tag': 'mychild', 'text': 'ab', 'tail': 'c'}])
    check_str(klon, element, '<mynode><mychild>ab</mychild>c</mynode>')

@parametrize_implementations
def test_none_values_are_noop(klon):
    element = klon.build_etree('mynode', ['mychild', 'a', None, 'b'], None, 'c')
    check(element, 'mynode', children=[{'tag': 'mychild', 'text': 'ab', 'tail': 'c'}])
    check_str(klon, element, '<mynode><mychild>ab</mychild>c</mynode>')

#----------------------------------------------------------------------------------------------------------------------------------



#!/usr/bin/env python

#----------------------------------------------------------------------------------------------------------------------------------

# 2+3 compatibility
from __future__ import unicode_literals

# standards
from os import path
import re
import setuptools

#----------------------------------------------------------------------------------------------------------------------------------

with open(path.join(path.dirname(__file__), 'README.md'), 'rb') as file_in:
    long_description = file_in.read().decode('UTF-8')

with open(path.join(path.dirname(__file__), 'klon', 'version.py'), 'rb') as file_in:
    klon_version = re.search(
        r'KLON_VERSION = \'(.+)\'',
        file_in.read().decode('UTF-8'),
    ).group(1)

setuptools.setup(
    name='klon',
    version=klon_version,
    author='Herv\u00e9 Saint-Amand',
    author_email='klon@saintamh.org',
    description='Utilities for building ElementTrees',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/saintamh/klon/',
    packages=setuptools.find_packages(),
    install_requires=[],
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Topic :: Text Processing :: Markup :: HTML',
        'Topic :: Text Processing :: Markup :: XML',
    ],
)

#----------------------------------------------------------------------------------------------------------------------------------
